@@ -3,7 +3,7 @@
 
  Part of the Routino routing software.
  ******************/ /******************
- This file Copyright 2011-2013 Andrew M. Bishop
+ This file Copyright 2011-2014 Andrew M. Bishop
 
  This program is free software: you can redistribute it and/or modify
  it under the terms of the GNU Affero General Public License as published by
@@ -77,8 +77,9 @@ void StartPruning(NodesX *nodesx,SegmentsX *segmentsx,WaysX *waysx)
  /* Allocate the array of next segment */
 
  segmentsx->next1=(index_t*)calloc(segmentsx->number,sizeof(index_t));
+ log_malloc(segmentsx->next1,segmentsx->number*sizeof(index_t));
 
- logassert(segmentsx->next1,"Failed to allocate memory (try using slim mode?)"); /* Check malloc() worked */
+ logassert(segmentsx->next1,"Failed to allocate memory (try using slim mode?)"); /* Check calloc() worked */
 
  /* Open the file read-only */
 
@@ -129,9 +130,11 @@ void StartPruning(NodesX *nodesx,SegmentsX *segmentsx,WaysX *waysx)
 void FinishPruning(NodesX *nodesx,SegmentsX *segmentsx,WaysX *waysx)
 {
  if(segmentsx->next1)
+   {
+    log_free(segmentsx->next1);
     free(segmentsx->next1);
-
- segmentsx->next1=NULL;
+    segmentsx->next1=NULL;
+   }
 }
 
 
@@ -184,14 +187,17 @@ void PruneIsolatedRegions(NodesX *nodesx,SegmentsX *segmentsx,WaysX *waysx,dista
  connected=AllocBitMask(segmentsx->number);
  region   =AllocBitMask(segmentsx->number);
 
+ log_malloc(connected,LengthBitMask(segmentsx->number)*sizeof(BitMask));
+ log_malloc(region   ,LengthBitMask(segmentsx->number)*sizeof(BitMask));
+
  logassert(connected,"Failed to allocate memory (try using slim mode?)"); /* Check AllocBitMask() worked */
- logassert(region,"Failed to allocate memory (try using slim mode?)");    /* Check AllocBitMask() worked */
+ logassert(region   ,"Failed to allocate memory (try using slim mode?)"); /* Check AllocBitMask() worked */
 
  regionsegments=(index_t*)malloc((nallocregionsegments=1024)*sizeof(index_t));
  othersegments =(index_t*)malloc((nallocothersegments =1024)*sizeof(index_t));
 
  logassert(regionsegments,"Failed to allocate memory (try using slim mode?)"); /* Check malloc() worked */
- logassert(othersegments,"Failed to allocate memory (try using slim mode?)");  /* Check malloc() worked */
+ logassert(othersegments ,"Failed to allocate memory (try using slim mode?)"); /* Check malloc() worked */
 
  /* Loop through the transport types */
 
@@ -390,6 +396,9 @@ void PruneIsolatedRegions(NodesX *nodesx,SegmentsX *segmentsx,WaysX *waysx,dista
 
  /* Unmap from memory / close the files */
 
+ log_free(region);
+ log_free(connected);
+
  free(region);
  free(connected);
 
@@ -593,7 +602,7 @@ void PruneShortSegments(NodesX *nodesx,SegmentsX *segmentsx,WaysX *waysx,distanc
           WayX *wayx1,*wayx2,*wayx3;
           NodeX *nodex2,*nodex3,*newnodex;
           index_t newnode;
-          int join12=1,join23=1;
+          int join12=1,join23=1,same13=1;
 
           /* Check if pruning would collapse a loop */
 
@@ -694,10 +703,35 @@ void PruneShortSegments(NodesX *nodesx,SegmentsX *segmentsx,WaysX *waysx,distanc
 
           PutBackNodeX(nodesx,newnodex);
 
-          /* Modify segments */
+          /* Modify segments - update the distances */
 
-          segmentx1->distance+=DISTANCE(segmentx2->distance)/2;
-          segmentx3->distance+=DISTANCE(segmentx2->distance)-DISTANCE(segmentx2->distance)/2;
+          if(!IsOneway(segmentx1) && !IsOneway(segmentx3))
+             ;
+          else if(IsOneway(segmentx1) && IsOneway(segmentx3))
+            {
+             if(IsOnewayTo(segmentx1,node3) && !IsOnewayFrom(segmentx3,node3)) /* S1 is one-way but S3 doesn't continue */
+                same13=0;
+
+             if(IsOnewayFrom(segmentx1,node3) && !IsOnewayTo(segmentx3,node3)) /* S1 is one-way but S3 doesn't continue */
+                same13=0;
+            }
+          else
+             same13=0;
+
+          if(WaysCompare(&wayx1->way,&wayx3->way))
+             same13=0;
+
+          if(same13)
+            {
+             segmentx1->distance+=DISTANCE(segmentx2->distance)/2;
+             segmentx3->distance+=DISTANCE(segmentx2->distance)-DISTANCE(segmentx2->distance)/2;
+            }
+          else if(join12)
+             segmentx1->distance+=DISTANCE(segmentx2->distance);
+          else /* if(join23) */
+             segmentx3->distance+=DISTANCE(segmentx2->distance);
+
+          /* Modify segments - update the segments */
 
           if(segmentx1->node1==node1)
             {
@@ -884,12 +918,14 @@ void PruneStraightHighwayNodes(NodesX *nodesx,SegmentsX *segmentsx,WaysX *waysx,
 
  checked=AllocBitMask(nodesx->number);
 
+ log_malloc(checked,LengthBitMask(nodesx->number)*sizeof(BitMask));
+
  logassert(checked,"Failed to allocate memory (try using slim mode?)"); /* Check AllocBitMask() worked */
 
  nodes   =(index_t*)malloc((nalloc=1024)*sizeof(index_t));
  segments=(index_t*)malloc( nalloc      *sizeof(index_t));
 
- logassert(nodes,"Failed to allocate memory (try using slim mode?)");    /* Check malloc() worked */
+ logassert(nodes   ,"Failed to allocate memory (try using slim mode?)"); /* Check malloc() worked */
  logassert(segments,"Failed to allocate memory (try using slim mode?)"); /* Check malloc() worked */
 
  lats=(double*)malloc(nalloc*sizeof(double));
@@ -1220,6 +1256,7 @@ void PruneStraightHighwayNodes(NodesX *nodesx,SegmentsX *segmentsx,WaysX *waysx,
 
  /* Unmap from memory / close the files */
 
+ log_free(checked);
  free(checked);
 
  free(nodes);
