@@ -89,7 +89,7 @@ Select start and end points (click on the marker icons below), select routing pr
 <td>(DE)
 <td><input name="language" type="radio" value="de" onchange="formSetLanguage();" >
 <tr>
-<td><a id="lang_fr_url" href="router.html.fr" title="Francais">Francais</a>
+<td><a id="lang_fr_url" href="router.html.fr" title="Page web en français">Français</a>
 <td>(FR)
 <td><input name="language" type="radio" value="fr" onchange="formSetLanguage();" >
 <tr>
@@ -145,9 +145,15 @@ Select start and end points (click on the marker icons below), select routing pr
 </div>
 </div>
 </div>
-<div id="waypoints-buttons" class="center">
-<input type="button" title="Reverse order of waypoints" value="Reverse order" onmousedown="markersReverse();">
-<input type="button" title="Add a new waypoint to make a loop" value="Close loop" onmousedown="markersLoop();">
+<div id="waypoints-buttons">
+<table>
+<tr><td>Close loop: <td><input type="checkbox" name="loop" onchange="formSetLoopReverse('loop' );">
+<tr><td>Reverse order:<td><input type="checkbox" name="reverse" onchange="formSetLoopReverse('reverse');">
+</table>
+<div class="waypoint-buttons" style="display: inline-block;">
+<img src="icons/waypoint-loop.png" title="Add a new waypoint to make a loop" onmousedown="markersLoop();">
+<img src="icons/waypoint-reverse.png" title="Reverse order of waypoints" onmousedown="markersReverse();">
+</div>
 </div>
 </div>
 </div>
