@@ -83,7 +83,10 @@ var legal={"^lon"             : "^[-0-9.]+$",
            "^width"           : "^[0-9.]+$",
            "^length"          : "^[0-9.]+$",
 
-           "^language"        : "^[-a-zA-Z]+$"};
+           "^language"        : "^[-a-zA-Z]+$",
+
+           "^reverse"         : "(1|0|true|false|on|off)",
+           "^loop"            : "(1|0|true|false|on|off)"};
 
 var args={};
 
@@ -221,6 +224,16 @@ function form_init()            // called from router.html
        searchfield.attachEvent("keyup", searchOnReturnKey); // Internet Explorer
    }
 
+ if(args["loop"] !== undefined)
+    formSetLoopReverse("loop",args["loop"]);
+ else
+    formSetLoopReverse("loop",false);
+
+ if(args["reverse"] !== undefined)
+    formSetLoopReverse("reverse",args["reverse"]);
+ else
+    formSetLoopReverse("reverse",false);
+
  // Update the transport type with the URL settings which updates all HTML forms to defaults.
 
  var transport=routino.transport;
@@ -481,10 +494,9 @@ function formSetRestriction(type,value) // called from router.html (with one arg
  if(type=="oneway" || type=="turns")
    {
     if(value === undefined)
-       routino.profile_restrictions[type][routino.transport]=document.forms["form"].elements["restrict-" + type].checked;
-    else
-       document.forms["form"].elements["restrict-" + type].checked=value;
+       value=document.forms["form"].elements["restrict-" + type].checked;
 
+    document.forms["form"].elements["restrict-" + type].checked=value;
     routino.profile_restrictions[type][routino.transport]=value;
    }
  else if(type=="weight")
@@ -620,6 +632,26 @@ function formSetSearch(marker,search) // called from event handler linked to rou
 }
 
 
+//
+// Change of loop or reverse option in the form
+//
+
+function formSetLoopReverse(type,value) // called from router.html (with one argument)
+{
+ if(value === undefined)
+    value=document.forms["form"].elements[type].checked;
+
+ document.forms["form"].elements[type].checked=value;
+
+ if(type == "loop")
+    routino.loop=value;
+ else
+    routino.reverse=value;
+
+ updateURLs();
+}
+
+
 //
 // Format a number in printf("%.5f") format.
 //
@@ -677,6 +709,12 @@ function buildURLArguments(lang)
     if(routino.profile_restrictions[key][routino.transport]!=routino_default.profile_restrictions[key][routino.transport])
        url=url + ";" + key + "=" + routino.profile_restrictions[key][routino.transport];
 
+ if(routino.loop)
+    url=url + ";loop=true";
+
+ if(routino.reverse)
+    url=url + ";reverse=true";
+
  if(lang && routino.language)
     url=url + ";language=" + routino.language;
 
@@ -1382,12 +1420,21 @@ function markerLocate(marker)   // called from router.html
 {
  clearSearchResult(marker);
 
+ function success(position)
+ {
+  formSetCoords(marker,position.coords.longitude,position.coords.latitude);
+  markerAddMap(marker);
+ }
+
+ function failure(error)
+ {
+  alert("Error: " + error.message);
+ }
+
  if(navigator.geolocation)
-    navigator.geolocation.getCurrentPosition(
-                                             function(position) {
-                                              formSetCoords(marker,position.coords.longitude,position.coords.latitude);
-                                              markerAddMap(marker);
-                                             });
+    navigator.geolocation.getCurrentPosition(success,failure);
+ else
+    alert("Error: Geolocation unavailable");
 }
 
 
@@ -1473,8 +1520,8 @@ function markerSetClearHome(marker,home)
 
  if(home)
    {
-    homelat=routino.point[marker].lat;
-    homelon=routino.point[marker].lon;
+    homelat=format5f(routino.point[marker].lat);
+    homelon=format5f(routino.point[marker].lon);
 
     cookie="Routino-home=lon:" + homelon + ":lat:" + homelat;
 
@@ -2133,9 +2180,9 @@ function getRouteSuccess(response)
     if(p!=point-1)
        points[p].html += "<tr><td>" + points[p].total;
 
-    result=result + "<tr onmouseover='highlight(\"" + routing_type + "\"," + p + ",\"show\")'>" +
-                    "<td onclick='highlight(\"" + routing_type + "\"," + p + ",\"zoom\")'" +
-                    " class='distance' title='" + points[p].distance + "'>#" + (p+1) +
+    result=result + "<tr onmouseover='highlight(\"" + routing_type + "\"," + p + ",\"show\")' " +
+                    "onclick='highlight(\"" + routing_type + "\"," + p + ",\"zoom\")'>" +
+                    "<td class='distance' title='" + points[p].distance + "'>#" + (p+1) +
                     "<td class='highway'>" + points[p].highway;
    }
 
