@@ -86,7 +86,7 @@ Only data relevant for routing is displayed and some will therefore be excluded.
 <td><a id="lang_de_url" href="visualiser.html.de" title="Deutsche Webseite">Deutsch</a>
 <td>(DE)
 <tr>
-<td><a id="lang_fr_url" href="visualiser.html.fr" title="Francais">Francais</a>
+<td><a id="lang_fr_url" href="visualiser.html.fr" title="Page web en français">Français</a>
 <td>(FR)
 <tr>
 <td><a id="lang_hu_url" href="visualiser.html.hu" title="Magyar weblap">Magyar</a>
